﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Windows.Threading;
using CygNet_Call_Center.Models;
using CygNet_Call_Center.Services;

namespace CygNet_Call_Center
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window, INotifyPropertyChanged
    {
        #region Properties

        private Conversation _selectedConversation;
        public Conversation SelectedConversation
        {
            get { return _selectedConversation; }
            set
            {
                _selectedConversation = value;
                UpdateForSelectedConversation();
                OnPropertyChanged("SelectedConversation");
            }
        }

        private ObservableCollection<Conversation> _conversations;
        public ObservableCollection<Conversation> Conversations
        {
            get { return _conversations; }
            set
            {
                _conversations = value;
                OnPropertyChanged("Conversations");
            }
        }

        private ObservableCollection<CygNetData> _cygnetData;
        public ObservableCollection<CygNetData> CygNetData
        {
            get { return _cygnetData; }
            set
            {
                _cygnetData = value;
                OnPropertyChanged("CygNetData");
            }
        }
        
        private TwilioHost _twilio;
        private CygNetService _cygnet;
        private DispatcherTimer _updateMsgTimer = new System.Windows.Threading.DispatcherTimer();
        private Dictionary<String, String> _phoneNumbers = new Dictionary<string, string>();

        #endregion

        public MainWindow()
        {
            InitializeComponent();
            Conversations = new ObservableCollection<Conversation>();
            _twilio = new TwilioHost();
            _cygnet = new CygNetService();
            CygNetData = new ObservableCollection<CygNetData>();

            _phoneNumbers.Add("+18051231234", "John Little");
            _phoneNumbers.Add("+18051231234", "Jack Smith");

            //Set update timer
            _updateMsgTimer.Tick += new EventHandler(GetNewMessages);
            _updateMsgTimer.Interval = new TimeSpan(0, 0, 5);

            //Manually get messages
            GetNewMessages(null, null);
        }

        private void ConsumeMessages(List<Twilio.Message> Messages)
        {
            bool UpdateScreen = false;
            foreach (var msg in Messages)
            {
                string phoneNumber;
                Common.ConversationStatus newStatus;
                if (msg.From == _twilio._twilioPhoneNumber)
                {
                    newStatus = Common.ConversationStatus.Sent;
                    phoneNumber = msg.To;
                }
                else
                {
                    newStatus = Common.ConversationStatus.Received;
                    phoneNumber = msg.From;
                }

                var iter = Conversations.Where(x => x.PhoneNumber == phoneNumber).ToList();
                if (iter.Count() >= 1)
                {
                    iter.First().Summary = msg.Body;
                    iter.First().Status = newStatus;
                    iter.First().Messages.Add(msg);
                }
                else
                {
                    Conversations.Add(new Conversation { Name = (_phoneNumbers.ContainsKey(phoneNumber) ? _phoneNumbers[phoneNumber] : "Unknown"), 
                                                         PhoneNumber = phoneNumber, 
                                                         Summary = msg.Body,
                                                         Status = newStatus,
                                                         Messages = new ObservableCollection<Twilio.Message>() { msg }});
                }

                UpdateScreen = true;
            }

            if (UpdateScreen)
                UpdateForSelectedConversation();
        }

        //Called by a timer
        private void GetNewMessages(object sender, EventArgs e)
        {
            _updateMsgTimer.Stop();
            ConsumeMessages(_twilio.GetMessages());
            _updateMsgTimer.Start();
        }

        public event PropertyChangedEventHandler PropertyChanged;
        public void OnPropertyChanged(string propName)
        {
            if (PropertyChanged != null)
            {
                PropertyChanged(this, new PropertyChangedEventArgs(propName));
            }
        }

        private void OnConversationSelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            DataGrid grid = (DataGrid)sender;
            if (grid.SelectedItem == null)
            {
                SelectedConversation = null;
            }
            else
            {
                SelectedConversation = (Conversation)grid.SelectedItem;
            }
        }

        private void UpdateForSelectedConversation()
        {
            if (SelectedConversation != null)
            {
                TxtMsgDetails.Text = "";
                foreach (var msg in SelectedConversation.Messages)
                {
                    if (msg.To == _twilio._twilioPhoneNumber)
                        TxtMsgDetails.Inlines.Add(new Run(msg.Body) { FontWeight = FontWeights.Bold });
                    else
                        TxtMsgDetails.Inlines.Add(msg.Body);
                    TxtMsgDetails.Inlines.Add(Environment.NewLine);
                }
                MsgScroll.ScrollToEnd();

                //Update the points list if there is a tag
                CygNetData = _cygnet.GetPoints(SelectedConversation.FacilityTag);
            }
            else
            {
                TxtMsgDetails.Text = "";
                TxtNewText.Text = "";
                if (CygNetData != null)
                    CygNetData.Clear();
            }
        }

        private void ChooseFacilityTag(object sender, RoutedEventArgs e)
        {
            if (SelectedConversation != null)
            {
                FacilityChooser dlg = new FacilityChooser();

                dlg.CygNetFacilities = _cygnet.GetFacilities(TxtCygNetSiteName.Text);
                dlg.ShowDialog();
                if (dlg.SelectedFacility == null)
                    return;

                SelectedConversation.FacilityTag = dlg.SelectedFacility.Tag;
                CygNetData = _cygnet.GetPoints(dlg.SelectedFacility.Tag);
            }
        }

        private void SendText(object sender, RoutedEventArgs e)
        {
            if (SelectedConversation != null)
            {
                _twilio.SendText(SelectedConversation.PhoneNumber, TxtNewText.Text);
                TxtNewText.Text = "";

                GetNewMessages(null, null);
            }
        }

        void CygNetDataGrid_PreviewMouseLeftButtonDown(object sender, MouseButtonEventArgs e)
        {
            if (CygNetDataGrid.CurrentColumn == null || CygNetDataGrid.CurrentItem == null)
                return;

            var cellContent = (CygNetDataGrid.CurrentColumn.GetCellContent(CygNetDataGrid.CurrentItem));
            if (cellContent != null)
            {
                var value = (cellContent as TextBlock).Text;

                DragDrop.DoDragDrop(CygNetDataGrid, value, DragDropEffects.Copy);
            }
        }

        private void CreateConversation(object sender, RoutedEventArgs e)
        {
            CreateConversation dlg = new CreateConversation();
            dlg.ShowDialog();
            if (dlg.NewConversation == null)
                return;

            if (Conversations.Where(x => x.PhoneNumber == dlg.NewConversation.PhoneNumber).Count() > 0)
            {
                MessageBox.Show("There is already a conversation using that phone number");
            }
            else
            {
                Conversations.Add(dlg.NewConversation);
            }
        }
        
        private void DeleteConversation(object sender, RoutedEventArgs e)
        {
            if (SelectedConversation == null)
                return;

            var ret = MessageBox.Show("Are you sure you want to delete the conversation?", "Confirm Delete", MessageBoxButton.YesNo);
            if (ret != MessageBoxResult.Yes)
                return;

            _twilio.DeleteMessages(SelectedConversation.Messages);
            Conversations.Remove(SelectedConversation);
            SelectedConversation = null;
        }

        private void SaveConversation(object sender, RoutedEventArgs e)
        {
            if (SelectedConversation == null)
                return;

            CreateNote dlg = new CreateNote();
            bool? bRet = dlg.ShowDialog();

            if (bRet != null && bRet.Value == true)
            {
                _cygnet.CreateNote(TxtCygNetSiteName.Text, SelectedConversation, dlg.SummaryText, Convert.ToDateTime(dlg.BeginRange), Convert.ToDateTime(dlg.EndRange));
            }
        }

        private void CallPhones(object sender, RoutedEventArgs e)
        {
            var numbers = Conversations.Select(x => x.PhoneNumber).ToList();
            _twilio.CallNumbersInParallel(numbers);
        }
    }
}
