﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Xml;
using System.Reflection;
using CygNet_Call_Center.Models;
using CXSCRIPTLib;
using CxNoteLib;

namespace CygNet_Call_Center.Services
{
    class CygNetService
    {
        private GlobalFunctions _globalFunctions;
        private Points _points;
        private Facilities _facilities;
        private NoteClient _noteClient;

        public CygNetService()
        {
            InitialiseCygNetComs();
        }

        public ObservableCollection<CygNetFacility> GetFacilities(String CvsSiteService)
        {
            var facilities = new ObservableCollection<CygNetFacility>();
            string facilitySiteService = _globalFunctions.GetAssociatedServiceName(CvsSiteService, "FAC");

            object facTags;
            IEnumerable<String> facilityTagList = null;

            _facilities.GetFacilityTagList(facilitySiteService, "", out facTags);
            if (facTags != null)
            {
                facilityTagList = ((object[])facTags).Cast<string>();
            }

            _facilities.AddFacilitiesArray(facTags);
            _facilities.UpdateNow();

            foreach (var facTag in facilityTagList)
            {
                try
                {
                    var fac = new CygNetFacility();
                    fac.Tag = _facilities.GetFacilityAttribute(facTag, "facility_tag") ?? "<missing>";
                    fac.Name = _facilities.GetFacilityAttribute(facTag, "facility_type") ?? "<missing>";
                    fac.Description = _facilities.GetFacilityAttribute(facTag, "facility_desc") ?? "<missing>";

                    facilities.Add(fac);
                }
                catch { }
            }

            return facilities;
        }

        public ObservableCollection<CygNetData> GetPoints(String FacilityTag)
        {
            if (FacilityTag == null)
                return null;

            int nIndex = FacilityTag.IndexOf("::");
            if (nIndex <= 0)
                return null;

            String CvsSiteService = FacilityTag.Substring(0, nIndex);
            String FacilityId = FacilityTag.Substring(nIndex + 2);

            //Get the list of point records for this facility ID
            var pntSiteService = _globalFunctions.GetAssociatedServiceName(CvsSiteService, "PNT");

            object pointTags = null;
            _points.GetPointTagList(pntSiteService, String.Format("facility={0}", FacilityId), true, out pointTags);

            IEnumerable<String> pointTagList = null;
            if (pointTags != null)
            {
                pointTagList = ((object[])pointTags).Cast<string>();
            }


            //Add properties required to XML
            var DataProperties = typeof(CygNetData).GetProperties();
            string strXmlReq;

            strXmlReq = "<cygPtInfo>" +
                            "<Parameters>";
            foreach (PropertyInfo prop in DataProperties)
            {
                strXmlReq += "<" + prop.Name.ToLower() + "/>";
            }
            strXmlReq += "</Parameters>" +
                             "<Points>";

            //Add Tags to XML
            int nNodeIndex = 1;
            foreach (var tag in pointTagList)
            {
                strXmlReq = strXmlReq + "<node" + nNodeIndex++ + " cygTag=\"" + tag + "\"" + "/>";
            }

            strXmlReq += "</Points>" +
                        "</cygPtInfo>";

            // add the points to the Points object and tell it to update
            _points.AddPointsArray(pointTagList.ToArray(), false);
            _points.UpdateNow(CXSCRIPTLib.UpdateNowType.updateCvsAndPcs);

            // now get the XML
            string strXmlResp = _points.GetPointsXML(strXmlReq);

            XmlDocument doc = new XmlDocument();
            doc.LoadXml(strXmlResp);

            // parse XML with XPath query
            var nodes = doc.SelectNodes("//Points");

            var data = new ObservableCollection<CygNetData>();
            foreach (XmlNode node in nodes)
            {
                foreach (XmlNode point in node.ChildNodes)
                {
                    var dataRow = new CygNetData();

                    foreach (PropertyInfo prop in DataProperties)
                    {
                        var item = point.Attributes.GetNamedItem(prop.Name.ToLower());
                        if (item != null)
                            prop.SetValue(dataRow, item.Value);
                    }

                    data.Add(dataRow);
                }
            }

            return data;
        }

        public void CreateNote(String CvsSiteService, Conversation conversation, String Summary, DateTime BeginRange, DateTime? EndRange)
        {
            var noteSiteService = _globalFunctions.GetAssociatedServiceName(CvsSiteService, "NOTE");

            var note = new Note();
            note.Summary = Summary;
            note.StartDateTime = BeginRange;
            if (EndRange != null)
            {
                note.EndDateTime = EndRange.Value;
            }

            String noteBody = "";
            foreach (var msg in conversation.Messages)
            {
                if (msg.To == conversation.PhoneNumber)
                {
                    noteBody += "Operator: " + msg.Body;
                }
                else
                {
                    noteBody += conversation.Name + ": " + msg.Body;
                }
                noteBody += "\r\n\r\n";
            }
            note.Body = noteBody;

            //Set note associations
            var associations = new List<Object>();
            if (conversation.FacilityTag != null && conversation.FacilityTag.Length > 0)
            {
                var noteAssociation = new Association();
                noteAssociation.SetAssociation(conversation.FacilityTag, EAssociationType.eFACILITY);
                associations.Add(noteAssociation);
            }
            if (conversation.PointTag != null && conversation.PointTag.Length > 0)
            {
                var pointAssociation = new Association();
                pointAssociation.SetAssociation(conversation.PointTag, EAssociationType.ePOINT);
                associations.Add(pointAssociation);
            }
            note.Associations = associations.ToArray();

            _noteClient.Connect(noteSiteService);
            var ret = _noteClient.CreateNote(note);
        }

        private void InitialiseCygNetComs()
        {
            try
            {
                _points = _points ?? new Points();
                _facilities = _facilities ?? new Facilities();
                _noteClient = _noteClient ?? new NoteClient();

                if (_globalFunctions == null)
                {
                    _globalFunctions = new GlobalFunctions();
                    _globalFunctions.EnableLiveMode(true);
                }
            }
            catch { }
        }

    }
}
